@@ -42,7 +42,7 @@
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  
- $Id: bibparse.y,v 1.26 1999/08/16 09:38:29 gobry Exp $
+ $Id: bibparse.y,v 1.27 1999/08/27 12:36:24 gobry Exp $
 */
 
 /*  #include "parsername.h" */
@@ -59,7 +59,10 @@ extern int bibtex_parser_debug;
 static BibtexEntry *	entry	= NULL;
 static int		start_line, entry_start;
 static BibtexSource *	current_source;
-static GString *	error_string = NULL;
+static gchar *	        error_string = NULL;
+static GString *        tmp_string = NULL;
+
+static gboolean recovering = FALSE;
 
 static void 
 nop (void) { 
@@ -82,10 +85,6 @@ bibtex_analyzer_finish (BibtexSource * source)  {
 
     bibtex_parser_finish (source);
     
-    if (error_string)
-	g_string_free (error_string, TRUE);
-    
-    error_string = NULL;
     current_source = NULL;
 }
  
@@ -95,8 +94,8 @@ bibtex_analyzer_parse (BibtexSource * source) {
 
   g_return_val_if_fail (source != NULL, NULL);
 
-  if (! error_string) {
-      error_string = g_string_new (NULL);
+  if (! tmp_string) {
+      tmp_string = g_string_new (NULL);
   }
 
   current_source = source;
@@ -121,6 +120,15 @@ bibtex_analyzer_parse (BibtexSource * source) {
       source->line += entry->length;
       bibtex_entry_destroy (entry, TRUE);
       entry = NULL;
+
+      if (error_string) {
+	  bibtex_error (error_string);
+      }
+  }
+
+  if (error_string) {
+      g_free (error_string);
+      error_string = NULL;
   }
 
   return entry;
@@ -128,13 +136,17 @@ bibtex_analyzer_parse (BibtexSource * source) {
 
 void 
 bibtex_parser_error (char * s) {
+    if (error_string) {
+	g_free (error_string);
+    }
+
     if (current_source) {
-	bibtex_error ("%s:%d: %s", current_source->name,
-		      start_line + entry->length, s);
+	error_string = g_strdup_printf ("%s:%d: %s", current_source->name,
+					start_line + entry->length, s);
     }
     else {
-	bibtex_error ("%d: %s", 
-		      start_line + entry->length, s);
+	error_string = g_strdup_printf ("%d: %s", 
+					start_line + entry->length, s);
 
     }
 }
@@ -154,14 +166,17 @@ bibtex_parser_warning (char * s) {
 
 static void 
 bibtex_parser_start_error (char * s) {
+    if (error_string) {
+	g_free (error_string);
+    }
+
     if (current_source) {
-	bibtex_error ("%s:%d: %s", current_source->name,
-		      entry_start, s);
+	error_string = g_strdup_printf ("%s:%d: %s", current_source->name,
+					entry_start, s);
     }
     else {
-	bibtex_error ("%d: %s", 
-		      entry_start, s);
-
+	error_string = g_strdup_printf ("%d: %s", 
+					entry_start, s);
     }
 }
 
@@ -179,7 +194,7 @@ bibtex_parser_start_warning (char * s) {
 }
 
 
-#line 161 "bibparse.y"
+#line 176 "bibparse.y"
 typedef union{
     gchar * text;
     BibtexStruct * body;
@@ -260,10 +275,10 @@ static const short yyrhs[] = {    10,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-   190,   199,   208,   215,   222,   229,   236,   250,   256,   262,
-   274,   322,   340,   346,   359,   360,   375,   376,   388,   395,
-   402,   408,   418,   425,   433,   439,   446,   457,   464,   473,
-   483,   490
+   205,   214,   223,   230,   248,   266,   273,   287,   293,   299,
+   311,   359,   377,   383,   396,   397,   412,   413,   425,   432,
+   441,   447,   457,   464,   472,   478,   485,   496,   503,   512,
+   522,   529
 };
 #endif
 
@@ -884,7 +899,7 @@ yyreduce:
   switch (yyn) {
 
 case 1:
-#line 192 "bibparse.y"
+#line 207 "bibparse.y"
 {
     entry->type = g_strdup (yyvsp[-3].text);
     g_strdown (entry->type);
@@ -893,7 +908,7 @@ case 1:
 ;
     break;}
 case 2:
-#line 201 "bibparse.y"
+#line 216 "bibparse.y"
 { 
     entry->type = g_strdup (yyvsp[-3].text);
     g_strdown (entry->type);
@@ -902,60 +917,82 @@ case 2:
 ;
     break;}
 case 3:
-#line 210 "bibparse.y"
+#line 225 "bibparse.y"
 { 
     current_source->eof = TRUE; 
     YYABORT; 
 ;
     break;}
 case 4:
-#line 217 "bibparse.y"
+#line 232 "bibparse.y"
 {
-    bibtex_parser_start_error ("perhaps there is a missing coma");
-    YYABORT;
+    if (current_source->strict) {
+	bibtex_parser_start_error ("perhaps a missing coma");
+	YYABORT;
+    }
+    else {
+	bibtex_parser_start_warning ("perhaps a missing coma.");
+
+	entry->type = g_strdup (yyvsp[-3].text);
+	g_strdown (entry->type);
+
+	yyclearin;
+	YYACCEPT;
+    }
 ;
     break;}
 case 5:
-#line 224 "bibparse.y"
+#line 250 "bibparse.y"
 {
-    bibtex_parser_start_error ("perhaps there is a missing coma");
-    YYABORT;
+    if (current_source->strict) {
+	bibtex_parser_start_error ("perhaps a missing coma");
+	YYABORT;
+    }
+    else {
+	bibtex_parser_start_warning ("perhaps a missing coma");
+
+	entry->type = g_strdup (yyvsp[-3].text);
+	g_strdown (entry->type);
+
+	yyclearin;
+	YYACCEPT;
+    }
 ;
     break;}
 case 6:
-#line 231 "bibparse.y"
+#line 268 "bibparse.y"
 {
     bibtex_parser_start_error ("end of file during processing");
     YYABORT;
 ;
     break;}
 case 7:
-#line 238 "bibparse.y"
+#line 275 "bibparse.y"
 {
     bibtex_parser_start_error ("end of file during processing");
     YYABORT;
 ;
     break;}
 case 8:
-#line 252 "bibparse.y"
+#line 289 "bibparse.y"
 {
     nop ();
 ;
     break;}
 case 9:
-#line 258 "bibparse.y"
+#line 295 "bibparse.y"
 {
     nop ();
 ;
     break;}
 case 10:
-#line 264 "bibparse.y"
+#line 301 "bibparse.y"
 {
     nop ();
 ;
     break;}
 case 11:
-#line 276 "bibparse.y"
+#line 313 "bibparse.y"
 { 
     char * name;
     BibtexField * field;
@@ -966,8 +1003,8 @@ case 11:
 
     /* Get a new instance of a field name */
     if (field) {
-	g_string_sprintf (error_string, "field `%s' is already defined", yyvsp[-2].text); 
-	bibtex_parser_warning (error_string->str);
+	g_string_sprintf (tmp_string, "field `%s' is already defined", yyvsp[-2].text); 
+	bibtex_parser_warning (tmp_string->str);
 
 	bibtex_field_destroy (field, TRUE);
 	name = yyvsp[-2].text;
@@ -1003,7 +1040,7 @@ case 11:
 ;
     break;}
 case 12:
-#line 324 "bibparse.y"
+#line 361 "bibparse.y"
 { 
     entry_start = start_line + entry->length;
 
@@ -1016,23 +1053,23 @@ case 12:
 ;
     break;}
 case 13:
-#line 342 "bibparse.y"
+#line 379 "bibparse.y"
 { 
     yyval.body = bibtex_struct_append (yyvsp[-2].body, yyvsp[0].body);
 ;
     break;}
 case 14:
-#line 348 "bibparse.y"
+#line 385 "bibparse.y"
 {
     yyval.body = yyvsp[0].body;
 ;
     break;}
 case 15:
-#line 359 "bibparse.y"
+#line 396 "bibparse.y"
 { bibtex_parser_is_content = TRUE; ;
     break;}
 case 16:
-#line 362 "bibparse.y"
+#line 399 "bibparse.y"
 { 
     bibtex_parser_is_content = FALSE; 
     yyval.body = bibtex_struct_new (BIBTEX_STRUCT_SUB);
@@ -1042,11 +1079,11 @@ case 16:
 ;
     break;}
 case 17:
-#line 375 "bibparse.y"
+#line 412 "bibparse.y"
 { bibtex_parser_is_content = TRUE; ;
     break;}
 case 18:
-#line 378 "bibparse.y"
+#line 415 "bibparse.y"
 { 
     bibtex_parser_is_content = FALSE; 
     yyval.body = bibtex_struct_new (BIBTEX_STRUCT_SUB);
@@ -1056,40 +1093,42 @@ case 18:
 ;
     break;}
 case 19:
-#line 390 "bibparse.y"
+#line 427 "bibparse.y"
 { 
     yyval.body = bibtex_struct_new (BIBTEX_STRUCT_TEXT);
     yyval.body->value.text = g_strdup (yyvsp[0].text);
 ;
     break;}
 case 20:
-#line 397 "bibparse.y"
+#line 434 "bibparse.y"
 {
     yyval.body = bibtex_struct_new (BIBTEX_STRUCT_REF);
     yyval.body->value.ref = g_strdup (yyvsp[0].text);
+
+    g_strdown (yyval.body->value.ref);
 ;
     break;}
 case 21:
-#line 404 "bibparse.y"
+#line 443 "bibparse.y"
 { 
     yyval.body = yyvsp[0].body;
 ;
     break;}
 case 22:
-#line 410 "bibparse.y"
+#line 449 "bibparse.y"
 { 
     yyval.body = yyvsp[0].body;
 ;
     break;}
 case 23:
-#line 420 "bibparse.y"
+#line 459 "bibparse.y"
 { 
     yyval.body = bibtex_struct_new (BIBTEX_STRUCT_COMMAND);
     yyval.body->value.com = g_strdup (yyvsp[0].text + 1);
 ;
     break;}
 case 24:
-#line 427 "bibparse.y"
+#line 466 "bibparse.y"
 { 
     yyval.body = bibtex_struct_new (BIBTEX_STRUCT_SUB);
     yyval.body->value.sub->encloser = BIBTEX_ENCLOSER_BRACE;
@@ -1097,34 +1136,34 @@ case 24:
 ;
     break;}
 case 25:
-#line 435 "bibparse.y"
+#line 474 "bibparse.y"
 {
     yyval.body = bibtex_struct_new (BIBTEX_STRUCT_SPACE);
 ;
     break;}
 case 26:
-#line 441 "bibparse.y"
+#line 480 "bibparse.y"
 {
     yyval.body = bibtex_struct_new (BIBTEX_STRUCT_SPACE);
     yyval.body->value.unbreakable = TRUE;
 ;
     break;}
 case 27:
-#line 448 "bibparse.y"
+#line 487 "bibparse.y"
 {
     yyval.body = bibtex_struct_new (BIBTEX_STRUCT_TEXT);
     yyval.body->value.text = g_strdup (yyvsp[0].text);
 ;
     break;}
 case 28:
-#line 459 "bibparse.y"
+#line 498 "bibparse.y"
 { 
     yyval.body = bibtex_struct_new (BIBTEX_STRUCT_TEXT);
     yyval.body->value.text = g_strdup ("");
 ;
     break;}
 case 29:
-#line 466 "bibparse.y"
+#line 505 "bibparse.y"
 { 
     yyval.body = bibtex_struct_new (BIBTEX_STRUCT_TEXT);
     yyval.body->value.text = g_strdup ("\"");
@@ -1133,20 +1172,20 @@ case 29:
 ;
     break;}
 case 30:
-#line 475 "bibparse.y"
+#line 514 "bibparse.y"
 { 
     yyval.body = bibtex_struct_append (yyvsp[-1].body, yyvsp[0].body);
 ;
     break;}
 case 31:
-#line 485 "bibparse.y"
+#line 524 "bibparse.y"
 { 
     yyval.body = bibtex_struct_new (BIBTEX_STRUCT_TEXT);
     yyval.body->value.text = g_strdup ("");
 ;
     break;}
 case 32:
-#line 492 "bibparse.y"
+#line 531 "bibparse.y"
 { 
     yyval.body = bibtex_struct_append (yyvsp[-1].body, yyvsp[0].body);
 ;
@@ -1373,6 +1412,6 @@ yyerrhandle:
     }
   return 1;
 }
-#line 499 "bibparse.y"
+#line 538 "bibparse.y"
      
 	
