@@ -19,18 +19,19 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 # 
-# $Id: pybliographic.py,v 1.3 1999/08/26 08:07:49 gobry Exp $
+# $Id: pybliographic.py,v 1.4 1999/08/27 12:38:00 gobry Exp $
 
 
 from Pyblio.GnomeUI import Pybliographic
 from gtk import *
 from Pyblio import Base
+import os
 
 main = Pybliographic.Pybliographic (version)
 main.show ()
 
 if len (sys.argv) > 1:
-    main.open (sys.argv [1])
+    main.open (os.path.abspath (sys.argv [1]))
 
 mainloop ()
 
